//
//  MultiComp_TestAppDelegate.m
//  MultiComp Test
//
//  Created by Greg Bowman on 11/21/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "MultiComp_TestAppDelegate.h"

@implementation MultiComp_TestAppDelegate

@synthesize window;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	// Insert code here to initialize your application 
	[self setupRenderer:1];
}

-(void)terminateVideo {
	//Stop the timer
	[_renderTimer invalidate];
	[_renderTimer release];
	
	//Destroy the renderer
	[_renderer1 release];
	[_renderer2 release];
	
	//Destroy the OpenGL context
	[_openGLContext clearDrawable];
	[_openGLContext release];
	
	//Release main screen
	if(CGDisplayIsCaptured(kCGDirectMainDisplay)) {
		CGDisplayShowCursor(kCGDirectMainDisplay);
		CGDisplayRelease(kCGDirectMainDisplay);
	}
	
	//Release path
	[_filePath release];
}

-(void)setupRenderer:(int)screenNumber {
	CGDisplayErr err;
	CGDirectDisplayID dspys [[[NSScreen screens] count]];
	CGDisplayCount dspyCnt;
	
	//Load composition file path
	NSString *path = [[NSString alloc] initWithString:[[NSBundle mainBundle] pathForResource:@"Video Loader 2.qtz" ofType:nil]];
	[path retain];
	NSString *path2 = [[NSString alloc] initWithString:[[NSBundle mainBundle] pathForResource:@"Basic Sprite.qtz" ofType:nil]];
	[path2 retain];
	
	err = CGGetOnlineDisplayList ([[NSScreen screens] count], dspys, &dspyCnt); if (err != CGDisplayNoErr) {
		NSLog(@"Cannot get display list");
		[NSApp terminate:nil];
	}
	
	CGDisplayCapture(dspys[screenNumber]);
	_screenSize.width = CGDisplayPixelsWide(dspys[screenNumber]);
	_screenSize.height = CGDisplayPixelsHigh(dspys[screenNumber]);
	
	GLint value = 1;
	NSOpenGLPixelFormatAttribute	attributes[] = {
		NSOpenGLPFAFullScreen,
		NSOpenGLPFAScreenMask, CGDisplayIDToOpenGLDisplayMask(dspys[screenNumber]),
		NSOpenGLPFANoRecovery,
		NSOpenGLPFADoubleBuffer,
		NSOpenGLPFAAccelerated,
		NSOpenGLPFADepthSize, 24,
		(NSOpenGLPixelFormatAttribute) 0
	};
	NSOpenGLPixelFormat* format = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attributes] autorelease];
	
	
	//Create the fullscreen OpenGL context on the main screen (double-buffered with color and depth buffers)
	_openGLContext = [[NSOpenGLContext alloc] initWithFormat:format shareContext:nil];
	if(_openGLContext == nil) {
		NSLog(@"Cannot create OpenGL context");
		[NSApp terminate:nil];
	}
	[_openGLContext setFullScreen];
	[_openGLContext setValues:&value forParameter:kCGLCPSwapInterval];
	
	
	//renderer 1 = video player patch
	_renderer1 = [[QCRenderer alloc] initWithOpenGLContext:_openGLContext pixelFormat:format file:path];
	//QCComposition *q = [
//	_renderer1 == [[QCRenderer alloc] initOffScreenWithSize:NSMakeSize(360,240) colorSpace:NULL composition:[QCComposition compositionWithFile:path]];
	if(_renderer1 == nil) {
		NSLog(@"Cannot create QCRenderer");
		[NSApp terminate:nil];
	}
	
	//renderer 2 = sprite patch
	_renderer2 = [[QCRenderer alloc] initWithOpenGLContext:_openGLContext pixelFormat:format file:path2];
	if(_renderer2 == nil) {
		NSLog(@"Cannot create QCRenderer");
		[NSApp terminate:nil];
	}
	
	//connect r1 to r2
	[_renderer2 setValue:[_renderer1 valueForOutputKey:[[_renderer1 outputKeys] objectAtIndex:0] ofType:@"QCImage"] forInputKey:[[_renderer2 inputKeys] objectAtIndex:0]];
	
	//Create a timer which will regularly call our rendering method
	_renderTimer = [[NSTimer scheduledTimerWithTimeInterval:(1.0 / (NSTimeInterval)kRendererFPS) target:self selector:@selector(_render:) userInfo:nil repeats:YES] retain];

	
}

- (void) renderWithEvent:(NSEvent*)event
{
	NSTimeInterval			time = [NSDate timeIntervalSinceReferenceDate];
	NSPoint					mouseLocation;
	NSMutableDictionary*	arguments;
	
	//Let's compute our local time
	if(_startTime == 0) {
		_startTime = time;
		time = 0;
	}
	else
		time -= _startTime;
	
	//We setup the arguments to pass to the composition (normalized mouse coordinates and an optional event)
	mouseLocation = [NSEvent mouseLocation];
	mouseLocation.x /= _screenSize.width;
	mouseLocation.y /= _screenSize.height;
	arguments = [NSMutableDictionary dictionaryWithObject:[NSValue valueWithPoint:mouseLocation] forKey:QCRendererMouseLocationKey];
	if(event)
		[arguments setObject:event forKey:QCRendererEventKey];
	
	//Render a frame
	if(![_renderer1 renderAtTime:time arguments:arguments])
		NSLog(@"Rendering failed at time %.3fs", time);
	
	if(![_renderer2 renderAtTime:time arguments:arguments])
		NSLog(@"Rendering failed at time %.3fs", time);
	
	
	
	
	
	//Flush the OpenGL context to display the frame on screen
	[_openGLContext flushBuffer];
}

- (void) _render:(NSTimer*)timer
{
	//Simply call our rendering method, passing no event to the composition
	[self renderWithEvent:nil];
}


- (void) sendEvent:(NSEvent*)event
{
	//If the user pressed the [Esc] key, we need to exit
	if(([event type] == NSKeyDown) && ([event keyCode] == 0x35))
		[NSApp terminate:nil];
	
	//If the renderer is active and we have a meaningful event, render immediately passing that event to the composition
	if(_renderer1 && (NSEventMaskFromType([event type]) & kRendererEventMask))
		[self renderWithEvent:event];
	
	if(_renderer2 && (NSEventMaskFromType([event type]) & kRendererEventMask))
		[self renderWithEvent:event];
	//else
	//		[super sendEvent:event];
}

@end
